#![cfg(feature = "server")]
use anyhow::Result;
use libpt::log::{error, info};
use mio::{self, event::Event, net::TcpListener, Events, Interest, Poll, Token};
use std::time::Duration;
use threadpool::ThreadPool;

use crate::common::conf::Config;

pub mod listener;

const EVENT_CAPACITY: usize = 128;
const SERVER: Token = Token(0);

pub struct Server {
    cfg: Config,
    work: Poll,
    events: Events,
    pool: ThreadPool,
    pub timeout: Option<Duration>,
    server: TcpListener,
}

impl Server {
    pub fn build(cfg: Config) -> Result<Self> {
        let mut server = TcpListener::bind(cfg.addr)?;
        let poll: Poll = Poll::new()?;
        poll.registry()
            .register(&mut server, SERVER, Interest::READABLE | Interest::WRITABLE);
        let events: Events = Events::with_capacity(EVENT_CAPACITY);
        let pool = ThreadPool::new(cfg.threads);
        let timeout = Some(Duration::from_secs(5));
        Ok(Server {
            cfg,
            work: poll,
            events,
            pool,
            timeout,
            server,
        })
    }
    pub fn run(&mut self) -> Result<()> {
        loop {
            self.work.poll(&mut self.events, self.timeout)?;
            for event in self.events.iter() {
                let result = self.handle_event(event);
                if let Err(err) = result {
                    error!("Error while handling server event {:?}: {:?}", event, err);
                }
            }
        }
    }

    fn handle_event(&self, event: &Event) -> Result<()> {
        dbg!(event);
        let connection = match self.server.accept() {
            Ok(c) => c,
            Err(err) => {
                error!("error while handling connection: {:?}", err);
                return Err(err.into());
            }
        };
        self.pool.execute(move|| {
            dbg!(connection);
            info!("received a connection!");
        });
        Ok(())
    }
}
